#' Scorecards load metrics from verification suite
#'@description Scorecards function to load saved data files 
#'@param recipe .yml with the minimum information:
#'  system.filename a list of character strings defining the names of the 
#'system names following the archive.yml format from verification suite (to be added)
#'  reference.filename a list of character strings defining the names of the 
#'references following the archive.yml format from verification suite (to be added)
#'Pending to be test with more than one.
#'   var.filename a character string following the format from variable-dictionary.yml 
#'from verification suite (todo: multiple variables)
#'  start.year numeric indicating the start year of the reference period
#'  end.year numeric indicating the end year of the reference period
#'@param m a vector of character strings indicating the start months
#'@param input.path a character string indicating the path where metrics output files from 
#'verification suite where store (or any other compatible files)
## Todo:
## other parameters? forecast time in case only one session or few months want to be displayed 
#'@example
#'recipe_file <- "recipes/recipe_1_nadia_scorecards.yml"
#'recipe <- read_yaml(recipe_file)
#loaded_metrics <- SC_load_metrics(system.filename = list('ecmwfs5','dwds21'),
#                                 reference.filename = 'era5',
#                                 var.filename = 'tas',
#                                 start.year = 1993,
#                                 end.year = 2016,
#                                 metric = 'mean_bias',
#                                 start.months = sprintf("%02d", 1:12),
#       input.path = '/esarchive/scratch/nmilders/scorecards_data/input_data')

SC_load_metrics <- function(system.filename, reference.filename, 
                            var.filename, start.year, end.year,
                            metrics = NULL,
                            start.months = sprintf("%02d", 1:12), 
                            input.path) {
  
  ## recipe: read recipe to obtain data of system, variable, start date etc.
  ## m: start_months to load from metrics outputs files
  ## metrics
  if (is.null(metric)) {
    metric <- c('mean_bias', 'enscorr', 'rpss', 'crpss', 'enssprerr') ## TO DO: read metrics from input
  }
  ## system.filename
  if (!is.list(system.filename)) {
    if  (!is.character(system.filename)) {
      stop("Parameter 'system.filename' must be a character or a list.")
    }
  } 
  ## input.path: 
  if (is.null(input.path)) {
     stop("Parameter 'input.path' cannot be NULL.")
  }
  if (!is.character(input.path)) {
    stop("Parameter 'input.path must be a character string.")
  }
  if (length(input.path) > 1) {
    input.path <- input.path[1]
    warning("Parameter 'input.path' has length greater than 1 and only the ",
            "first element will be used.")
  }
  if (is.null(start.year)) {
    stop("Parameter 'start.year' cannot be NULL.")
  }
  if (is.null(end.year)) {
    stop("Parameter 'end.year' cannot be NULL.")
  }
  #system.filename <- list('ecmwfs5','dwds21') ## archive$System[[recipe$Analysis$Datasets$System$name]]$system_filename ## 'ecmwfs5' 
  #reference.filename <- archive$Reference[[recipe$Analysis$Datasets$Reference$name]]$reference_filename ## 'era5'
  #var.filename <- recipe$Analysis$Variables$name ## 'tas'
  ## start.year
  ## end.year
  
  period <- paste0(start.year, "-", end.year)
  
  all_metrics <- list()
  for (sys in 1:length(system.filename)) {
    by_reference <- list()
    for (ref in 1:length(reference.filename)) {
       met <- initialfunction_read_data( 
                                   input.path = input.path, # recipe$Run$output, 
                                   system.filename = system.filename[sys],
                                   reference.filename = reference.filename[ref],
                                   var.filename = var.filename,
                                   period = period, 
                                   start.months = start.months,
                                   metrics = metrics)
       by_reference <- append(by_reference, list(met)) #append(met_by_sys, list(met))
       names(by_reference)[[length(by_reference)]] <- reference.filename[ref]
    }
    all_metrics <- append(all_metrics, list(by_reference)) #append(met_by_sys, list(met))
    names(all_metrics)[[length(all_metrics)]] <- system.filename[sys]
  }
  return(all_metrics)
}


##### Function to load data files #####

# metric is a vector indicating the metrics to load using the names in the file 
# metric is null by default and is currently hardcoded 
# Test:
#test <- initialfunction_read_data('/esarchive/scratch/nmilders/scorecards_data/input_data',
#                          "ecmwfs5", "era5", "tas", "1993-2016", start.months = sprintf("%02d", 1:12),
#                          metric = c('mean_bias', 'enscorr', 'rpss', 'crpss', 'enssprerr'))
# @return metrics in the order of the parameter metrics

initialfunction_read_data <- function(input.path, system.filename, 
                                      reference.filename, var.filename, period, start.months, metrics){
  ## Load data for each system
    lon <- NULL
    lat <- NULL
    met_by_grid <- NULL
    archivo_dim <- NULL
    ## Load data for each start date
    for(m in start.months){
     ## netCDF data file path:
     print(paste0("Loading netCDF file: ", paste0(input.path, "/", system.filename, "/", var.filename,
                                                "/scorecards_", system.filename, "_", reference.filename, "_",
                                                 var.filename, "_", period, "_s", m, ".nc")))
     nc <- paste0(input.path, "/", system.filename, "/", var.filename, 
                           "/scorecards_", system.filename, "_", reference.filename, "_", 
                           var.filename, "_", period, "_s", m, #mod.pressure,
                           ".nc")
      ## Load data of each metric:
      met_by_sdate <- NULL 
      for (kk in 1:length(metric)) {
         met <- tryCatch({easyNCDF::NcToArray(nc, vars_to_read = metric[kk], unlist = T, 
                                    drop_var_dim = T)},
                          error = function(cond) {
                            print("Missing file for start date")
                            message(cond)
                            return(NA)
                          })
         ## Save data for each metric
         attributes(met)$variables <- NULL
         if (is.null(archivo_dim)) {
           archivo_dim <- names(dim(met))
         }
         met_by_sdate <- abind::abind(met_by_sdate, met, along = 4) 
      } ## close loop on metric
      ## Load latitude and longitude data as attribute of system
      if (is.null(lon)) {
        lon <- tryCatch({easyNCDF::NcToArray(nc, vars_to_read = 'longitude', unlist = T, drop_var_dim = T)})
        lat <- tryCatch({easyNCDF::NcToArray(nc, vars_to_read = 'latitude', unlist = T, drop_var_dim = T)})
      }
    ## Save metric data for each start date  
    met_by_grid <- abind::abind(met_by_grid, met_by_sdate, along = 5)
    } ## close loop on start months
    
  ## Save metric data by start date for each system  
  attributes(met_by_grid)$lon <- lon
  attributes(met_by_grid)$lat <- lat
  names(dim(met_by_grid)) <- c(archivo_dim, 'metric', 'sdate')           
  return(met_by_grid)
 } ## close function 

   
    


